@@ -67,7 +67,7 @@ class Component( ComponentLevel7 ):
     except AttributeError:
       raise NotElaboratedError()
 
-  def _collect_objects_local( s, filt ):
+  def _collect_objects_local( s, filt, sort_key = None ):
     assert s._dsl.constructed
     ret = set()
     stack = []
@@ -82,7 +82,10 @@ class Component( ComponentLevel7 ):
       # ONLY LIST IS SUPPORTED
       elif isinstance( u, list ):
         stack.extend( u )
-    return ret
+    if sort_key:
+      return sorted( ret, key = sort_key )
+    else:
+      return list(ret)
 
   def _flush_pending_value_connections( s ):
     if s._dsl._has_pending_value_connections:
@@ -163,8 +166,9 @@ class Component( ComponentLevel7 ):
       obj._dsl._my_indices  = indices
 
       obj._dsl.elaborate_top = top
-      top._dsl.elaborate_stack.append( obj )
+      obj._dsl.NamedObject_fields = set()
 
+      top._dsl.elaborate_stack.append( obj )
       NamedObject.__setattr__ = NamedObject.__setattr_for_elaborate__
       obj._construct()
       del NamedObject.__setattr__
@@ -266,6 +270,7 @@ class Component( ComponentLevel7 ):
       else:
         # Simply delete the attribute if it's merely a field
         delattr( parent, foo._dsl.my_name )
+        parent._dsl.NamedObject_fields.remove( foo._dsl.my_name )
 
       # Remove all components, signals, and method ports
       removed_components, removed_signals, removed_method_ports = \
@@ -536,17 +541,17 @@ class Component( ComponentLevel7 ):
     except AttributeError:
       raise NotElaboratedError()
 
-  def get_child_components( s ):
-    return s._collect_objects_local( lambda x: isinstance( x, Component ) )
+  def get_child_components( s, sort_key = None ):
+    return s._collect_objects_local( lambda x: isinstance( x, Component ), sort_key )
 
-  def get_input_value_ports( s ):
-    return s._collect_objects_local( lambda x: isinstance( x, InPort ) )
+  def get_input_value_ports( s, sort_key = None ):
+    return s._collect_objects_local( lambda x: isinstance( x, InPort ), sort_key )
 
-  def get_output_value_ports( s ):
-    return s._collect_objects_local( lambda x: isinstance( x, OutPort ) )
+  def get_output_value_ports( s , sort_key = None ):
+    return s._collect_objects_local( lambda x: isinstance( x, OutPort ), sort_key )
 
-  def get_wires( s ):
-    return s._collect_objects_local( lambda x: isinstance( x, Wire ) )
+  def get_wires( s , sort_key = None ):
+    return s._collect_objects_local( lambda x: isinstance( x, Wire ), sort_key )
 
   def get_update_blocks( s ):
     assert s._dsl.constructed
@@ -584,9 +589,9 @@ class Component( ComponentLevel7 ):
     except AttributeError:
       return s._collect_all_single( filt )
 
-  def get_local_object_filter( s, filt ):
+  def get_local_object_filter( s, filt, sort_key = None ):
     assert callable( filt )
-    return s._collect_objects_local( filt )
+    return s._collect_objects_local( filt, sort_key )
 
   def get_all_components( s ):
     try:
