@@ -15,10 +15,10 @@ class PlaceholderPass( BasePass ):
     if not isinstance(m, Placeholder) and hasattr(m, 'config_placeholder'):
       raise PlaceholderConfigError(m,
           "the given object is not a Placeholder but has `config_placeholder` attribute!")
-    
+
     if isinstance( m, Placeholder ):
       s.visit_placeholder( m )
-    for child in m.get_child_components():
+    for child in m.get_child_components(repr):
       s.__call__( child )
 
   def visit_placeholder( s, m ):
