@@ -25,14 +25,14 @@ from pymtl3.passes.errors import InvalidPassOptionValue
 from pymtl3.passes.PlaceholderConfigs import expand
 from pymtl3.passes.PlaceholderPass import PlaceholderPass
 from pymtl3.passes.rtlir import RTLIRDataType as rdt
+from pymtl3.passes.rtlir import RTLIRGetter
 from pymtl3.passes.rtlir import RTLIRType as rt
-from pymtl3.passes.rtlir import get_component_ifc_rtlir
 
 
 class VerilogPlaceholderPass( PlaceholderPass ):
 
   def visit_placeholder( s, m ):
-    irepr = get_component_ifc_rtlir( m )
+    irepr = RTLIRGetter(cache=False).get_component_ifc_rtlir( m )
 
     s.setup_default_configs( m, irepr )
 
@@ -53,7 +53,17 @@ class VerilogPlaceholderPass( PlaceholderPass ):
       # parameters. Note we need to make sure the infered top_module matches
       # the default translation result.
       if not cfg.top_module:
-        cfg.top_module = get_component_unique_name( irepr )
+        cfg.top_module = irepr.get_name()
+        cfg.unique_top_module = get_component_unique_name( irepr )
+        # if m.get_parent_object() is None:
+        #   # This is a top level module -- use the module name without the
+        #   # parameters because we will generate a different wrapper for top!
+        #   cfg.top_module = irepr.get_name()
+        # else:
+        #   # Otherwise use the full name
+        #   cfg.top_module = get_component_unique_name( irepr )
+      else:
+        cfg.unique_top_module = cfg.top_module
 
       # Only try to infer the name of Verilog source file if both
       # flist and the source file are not specified.
@@ -110,46 +120,82 @@ class VerilogPlaceholderPass( PlaceholderPass ):
 
   def pickle( s, m, cfg, irepr ):
     # In the namespace cfg:
-    #   pickled_source_file: path to the pickled Verilog source file
+    #   pickled_dependency_file: path to the pickled Verilog dependency file
+    #   pickled_wrapper_file: path to the pickled Verilog wrapper file
     #   pickled_top_module:  name of the top module in the pickled Verilog
 
-    pickled_source_file = f'{cfg.top_module}_pickled.v'
-    # if hasattr( m, 'explicit_module_name' ):
-    #   pickled_top_module = m.explicit_module_name
-    # # if cfg.explicit_module_name:
-    # #   pickled_top_module = cfg.explicit_module_name
-    # else:
-    pickled_top_module = f'{cfg.top_module}_wrapper'
+    pickled_dependency_file = f'{cfg.unique_top_module}_pickled_dependency.v'
+    pickled_wrapper_file = f'{cfg.unique_top_module}_pickled_wrapper.v'
+    pickled_top_module = f'{cfg.unique_top_module}_wrapper'
 
     orig_comp_name       = get_component_unique_name( irepr )
-    pickle_dependency    = s._get_dependent_verilog_modules( m, cfg, irepr )
+    pickle_dependency    = s._get_v_lib_files( m, cfg, irepr )
+    pickle_dependency   += s._get_dependent_verilog_modules( m, cfg, irepr )
     pickle_wrapper, tplt = s._gen_verilog_wrapper( m, cfg, irepr, pickled_top_module )
     def_symbol           = pickled_top_module.upper()
 
     # The directives are there to prevent potential duplicated definitions
-    pickle_template = dedent(
-        '''\
-            // This is a wrapper module that wraps PyMTL placeholder {orig_comp_name}
-            // This file was generated by PyMTL VerilogPlaceholderPass
-            `ifndef {def_symbol}
-            `define {def_symbol}
+    # pickle_template = dedent(
+    #     '''\
+    #         // This is a wrapper module that wraps PyMTL placeholder {orig_comp_name}
+    #         // This file was generated by PyMTL VerilogPlaceholderPass
+    #         `ifndef {def_symbol}
+    #         `define {def_symbol}
 
-            {pickle_dependency}
-            {pickle_wrapper}
+    #         {pickle_dependency}
+    #         {pickle_wrapper}
 
-            `endif /* {def_symbol} */
-        '''
-    )
+    #         `endif /* {def_symbol} */
+    #     '''
+    # )
 
-    cfg.pickled_source_file      = pickled_source_file
+    cfg.pickled_dependency_file  = pickled_dependency_file
+    cfg.pickled_wrapper_file     = pickled_wrapper_file
     cfg.pickled_top_module       = pickled_top_module
     cfg.pickled_wrapper_template = tplt
+    cfg.pickled_orig_file        = cfg.src_file
     cfg.def_symbol = def_symbol
     cfg.orig_comp_name = orig_comp_name
     cfg.pickle_dependency = pickle_dependency
 
-    with open( pickled_source_file, 'w' ) as fd:
-      fd.write( pickle_template.format( **locals() ) )
+    with open( pickled_dependency_file, 'w' ) as fd:
+      fd.write( pickle_dependency )
+
+    with open( pickled_wrapper_file, 'w' ) as fd:
+      fd.write( pickle_wrapper )
+
+  def _get_v_lib_files( s, m, cfg, irepr ):
+    orig_comp_name = get_component_unique_name( irepr )
+    tplt = dedent(
+        """\
+            // The source code below are included because they are specified
+            // as the v_lib Verilog placeholder option of component {orig_comp_name}.
+
+            // If you get a duplicated def error from files included below, please
+            // make sure they are included either through the v_lib option or the
+            // explicit `include statement in the Verilog source code -- if they
+            // appear in both then they will be included twice!
+
+            {v_libs}
+            // End of all v_lib files for component {orig_comp_name}
+
+        """
+    )
+    per_file_tplt = dedent(
+        """\
+            // File {filename} from v_lib option of component {orig_comp_name}
+            {file_content}
+        """
+    )
+    v_libs = []
+
+    for filename in cfg.v_libs:
+      with open(filename) as fd:
+        file_content = fd.read()
+        v_libs.append(per_file_tplt.format(**locals()))
+
+    v_libs = ''.join(v_libs)
+    return tplt.format(**locals())
 
   def _get_dependent_verilog_modules( s, m, cfg, irepr ):
     return s._import_sources( cfg, [cfg.src_file] )
@@ -165,11 +211,19 @@ class VerilogPlaceholderPass( PlaceholderPass ):
       parameters = cfg.params.items()
 
     # Port definitions of wrapper
-    ports = [
-      f"  {p.get_direction()} logic [{p.get_dtype().get_length()}-1:0]"\
-      f" {name}{'' if idx == len(rtlir_ports)-1 else ','}" \
-      for idx, (_, name, p) in enumerate(rtlir_ports) if name
-    ]
+    ports = []
+    for idx, (_, name, p, _) in enumerate(rtlir_ports):
+      if name:
+        if isinstance(p, rt.Array):
+          n_dim = p.get_dim_sizes()
+          s_dim = ''.join([f'[0:{idx-1}]' for idx in n_dim])
+          p = p.get_next_dim_type()
+        else:
+          s_dim = ''
+        ports.append(
+            f"  {p.get_direction()} logic [{p.get_dtype().get_length()}-1:0]"\
+            f" {name} {s_dim}{'' if idx == len(rtlir_ports)-1 else ','}"
+        )
 
     # The wrapper has to have an unused clk port to make verilator
     # VCD tracing work.
@@ -188,7 +242,7 @@ class VerilogPlaceholderPass( PlaceholderPass ):
     # Connections between top module and inner module
     connect_ports = [
       f"    .{name}( {name} ){'' if idx == len(rtlir_ports)-1 else ','}"\
-      for idx, (_, name, p) in enumerate(rtlir_ports) if name
+      for idx, (_, name, p, _) in enumerate(rtlir_ports) if name
     ]
 
     lines = [
