@@ -33,6 +33,9 @@ from pymtl3.passes.testcases import (
     CaseBits64PartSelUpblkComp,
     CaseBits64SextInComp,
     CaseBits64ZextInComp,
+    CaseBitSelOverBitSelComp,
+    CaseBitSelOverPartSelComp,
+    CaseBoolTmpVarComp,
     CaseConnectArrayBits32FooIfcComp,
     CaseConnectArrayNestedIfcComp,
     CaseConnectArrayStructAttrToOutComp,
@@ -65,7 +68,10 @@ from pymtl3.passes.testcases import (
     CaseLambdaConnectWithListComp,
     CaseNestedIfComp,
     CaseNestedStructPackedArrayUpblkComp,
+    CasePartSelOverBitSelComp,
+    CasePartSelOverPartSelComp,
     CasePassThroughComp,
+    CasePythonClassAttr,
     CaseReducesInx3OutComp,
     CaseSequentialPassThroughComp,
     CaseSizeCastPaddingStructPort,
@@ -518,6 +524,69 @@ CaseBits64PartSelUpblkComp = set_attributes( CaseBits64PartSelUpblkComp,
     '''
 )
 
+CasePythonClassAttr = set_attributes( CasePythonClassAttr,
+    'REF_UPBLK',
+    '''\
+        always_comb begin : upblk
+          out1 = 42;
+          out2 = 32'd1;
+        end
+    ''',
+    'REF_SRC',
+    '''\
+        module DUT
+        (
+          input logic [0:0] clk,
+          output logic [31:0] out1,
+          output logic [31:0] out2,
+          input logic [0:0] reset
+        );
+
+          always_comb begin : upblk
+            out1 = 42;
+            out2 = 32'd1;
+          end
+
+        endmodule
+    '''
+)
+
+CaseBoolTmpVarComp = set_attributes( CaseBoolTmpVarComp,
+    'REF_UPBLK',
+    '''\
+        always_comb begin : upblk
+          __tmpvar__upblk_matched = in_ == 0;
+          if ( __tmpvar__upblk_matched ) begin
+            out = 1;
+          end
+          else
+            out = 0;
+        end
+    ''',
+    'REF_SRC',
+    '''\
+        module DUT
+        (
+          input logic [0:0] clk,
+          input logic [31:0] in_,
+          output logic [31:0] out,
+          input logic [0:0] reset
+        );
+          logic [0:0] __tmpvar__upblk_matched;
+
+          always_comb begin : upblk
+            __tmpvar__upblk_matched = in_ == 0;
+            if ( __tmpvar__upblk_matched ) begin
+              out = 1;
+            end
+            else
+              out = 0;
+          end
+
+        endmodule
+    '''
+)
+
 CaseReducesInx3OutComp = set_attributes( CaseReducesInx3OutComp,
     'REF_UPBLK',
     '''\
@@ -1003,25 +1072,20 @@ CaseConstStructInstComp = set_attributes( CaseConstStructInstComp,
     'REF_UPBLK',
     '''\
         always_comb begin : upblk
-          out = in_.foo;
+          out = 32'd0;
         end
     ''',
     'REF_SRC',
     '''\
-        typedef struct packed {
-          logic [31:0] foo;
-        } Bits32Foo;
-
         module DUT
         (
           input logic [0:0] clk,
           output logic [31:0] out,
           input logic [0:0] reset
         );
-          localparam Bits32Foo in_ = { 32'd0 };
 
           always_comb begin : upblk
-            out = in_.foo;
+            out = 32'd0;
           end
 
         endmodule
@@ -1455,6 +1519,134 @@ CaseConnectSliceToOutComp = set_attributes( CaseConnectSliceToOutComp,
     '''
 )
 
+CaseBitSelOverBitSelComp = set_attributes( CaseBitSelOverBitSelComp,
+    'REF_PORT',
+    '''\
+        input logic [0:0] clk,
+        input logic [31:0] in_,
+        output logic [0:0] out,
+        input logic [0:0] reset
+    ''',
+    'REF_WIRE',
+    '',
+    'REF_CONST',
+    '',
+    'REF_CONN',
+    '''\
+        assign out = in_[1:1];
+    ''',
+    'REF_SRC',
+    '''\
+        module DUT
+        (
+          input logic [0:0] clk,
+          input logic [31:0] in_,
+          output logic [0:0] out,
+          input logic [0:0] reset
+        );
+
+          assign out = in_[1:1];
+
+        endmodule
+    '''
+)
+
+CaseBitSelOverPartSelComp = set_attributes( CaseBitSelOverPartSelComp,
+    'REF_PORT',
+    '''\
+        input logic [0:0] clk,
+        input logic [31:0] in_,
+        output logic [0:0] out,
+        input logic [0:0] reset
+    ''',
+    'REF_WIRE',
+    '',
+    'REF_CONST',
+    '',
+    'REF_CONN',
+    '''\
+        assign out = in_[0:0];
+    ''',
+    'REF_SRC',
+    '''\
+        module DUT
+        (
+          input logic [0:0] clk,
+          input logic [31:0] in_,
+          output logic [0:0] out,
+          input logic [0:0] reset
+        );
+
+          assign out = in_[0:0];
+
+        endmodule
+    '''
+)
+
+CasePartSelOverBitSelComp = set_attributes( CasePartSelOverBitSelComp,
+    'REF_PORT',
+    '''\
+        input logic [0:0] clk,
+        input logic [31:0] in_,
+        output logic [0:0] out,
+        input logic [0:0] reset
+    ''',
+    'REF_WIRE',
+    '',
+    'REF_CONST',
+    '',
+    'REF_CONN',
+    '''\
+        assign out = in_[1:1];
+    ''',
+    'REF_SRC',
+    '''\
+        module DUT
+        (
+          input logic [0:0] clk,
+          input logic [31:0] in_,
+          output logic [0:0] out,
+          input logic [0:0] reset
+        );
+
+          assign out = in_[1:1];
+
+        endmodule
+    '''
+)
+
+CasePartSelOverPartSelComp = set_attributes( CasePartSelOverPartSelComp,
+    'REF_PORT',
+    '''\
+        input logic [0:0] clk,
+        input logic [31:0] in_,
+        output logic [0:0] out,
+        input logic [0:0] reset
+    ''',
+    'REF_WIRE',
+    '',
+    'REF_CONST',
+    '',
+    'REF_CONN',
+    '''\
+        assign out = in_[0:0];
+    ''',
+    'REF_SRC',
+    '''\
+        module DUT
+        (
+          input logic [0:0] clk,
+          input logic [31:0] in_,
+          output logic [0:0] out,
+          input logic [0:0] reset
+        );
+
+          assign out = in_[0:0];
+
+        endmodule
+    '''
+)
+
 CaseConnectConstStructAttrToOutComp = set_attributes( CaseConnectConstStructAttrToOutComp,
     'REF_PORT',
     '''\
