@@ -383,6 +383,10 @@ class BehavioralRTLIRToVVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
         value = int(node._value)
       else:
         value = node._value
+      # value could be larger than what nbits can hold because it might
+      # be a negative number. Since verilator errors when that happens, we
+      # need to manually truncate the integer.
+      value = int(Bits(nbits, value))
       return f"{nbits}'d{value}"
 
     return f"{nbits}'( {value} )"
