@@ -21,8 +21,8 @@ def run_test( case, m ):
   m.elaborate()
   visitor = YosysBehavioralRTLIRToVVisitorL5(lambda x: x in verilog_reserved)
 
-  m.apply( BehavioralRTLIRGenPass() )
-  m.apply( BehavioralRTLIRTypeCheckPass() )
+  m.apply( BehavioralRTLIRGenPass( m ) )
+  m.apply( BehavioralRTLIRTypeCheckPass( m ) )
   upblks = m._pass_behavioral_rtlir_gen.rtlir_upblks
   m_all_upblks = m.get_update_blocks()
   assert len(m_all_upblks) == 1
