@@ -352,6 +352,57 @@ class CaseStructClosureGlobal:
       def upblk():
         s.out = foo.foo
 
+class CasePythonClassAttr:
+  class DUT( Component ):
+    def construct( s ):
+      class Enum:
+        int_attr = 42
+        bit_attr = Bits32(1)
+      s.out1 = OutPort( Bits32 )
+      s.out2 = OutPort( Bits32 )
+      @s.update
+      def upblk():
+        s.out1 = Enum.int_attr
+        s.out2 = Enum.bit_attr
+  TV_IN = _set()
+  TV_OUT = \
+  _check(
+      'out1', Bits32, 0,
+      'out2', Bits32, 1,
+  )
+  TEST_VECTOR = \
+  [
+      [  42,   1,  ],
+      [  42,   1,  ],
+  ]
+
+class CaseBoolTmpVarComp:
+  class DUT( Component ):
+    def construct( s ):
+      s.in_ = InPort( Bits32 )
+      s.out = OutPort( Bits32 )
+      @s.update
+      def upblk():
+        matched = s.in_ == 0
+        if matched:
+          s.out = 1
+        else:
+          s.out = 0
+  TV_IN = _set(
+      'in_', Bits32, 0
+  )
+  TV_OUT = \
+  _check(
+      'out', Bits32, 1,
+  )
+  TEST_VECTOR = \
+  [
+      [  0,   1,  ],
+      [  1,   0,  ],
+      [  1,   0,  ],
+      [  0,   1,  ],
+  ]
+
 class CaseReducesInx3OutComp:
   class DUT( Component ):
     def construct( s ):
@@ -1837,7 +1888,7 @@ class CaseConnectArrayBits32FooIfcComp:
   )
   TEST_VECTOR = \
   [
-      [ 1,      0,    1,      0, ], 
+      [ 1,      0,    1,      0, ],
       [ 0,     42,    0,     42, ],
       [ 1,     42,    1,     42, ],
       [ 1,     -1,    1,     -1, ],
@@ -1875,7 +1926,7 @@ class CaseConnectArrayNestedIfcComp:
   )
   TEST_VECTOR = \
   [
-      [ 1,  1,      0,    0,    1,  1,      0,    0,  1,  1,      0,    0,    1,  1,      0,    0, ], 
+      [ 1,  1,      0,    0,    1,  1,      0,    0,  1,  1,      0,    0,    1,  1,      0,    0, ],
       [ 1,  0,     42,    1,    1,  0,     42,    1,  1,  0,     42,    1,    1,  0,     42,    1, ],
       [ 1,  1,     42,    0,    1,  1,     42,    0,  1,  1,     42,    0,    1,  1,     42,    0, ],
       [ 1,  1,     -1,    1,    1,  1,     -1,    1,  1,  1,     -1,    1,    1,  1,     -1,    1, ],
@@ -2122,16 +2173,22 @@ class CaseGenericConditionalDriveComp:
                                 "s.out0 = " + str( s.out[0] ) +\
                                 "s.out1 = " + str( s.out[1] )
 
-#-------------------------------------------------------------------------
-# Test cases that contain generic translator errors
-#-------------------------------------------------------------------------
-
 class CaseBitSelOverBitSelComp:
   class DUT( Component ):
     def construct( s ):
       s.in_ = InPort( Bits32 )
       s.out = OutPort( Bits1 )
       connect( s.out, s.in_[1][0] )
+  TV_IN  = _set( 'in_', Bits32, 0 )
+  TV_OUT = _check( 'out', Bits1, 1 )
+  TEST_VECTOR = \
+  [
+      [ 1, 0 ],
+      [ 2, 1 ],
+      [ 3, 1 ],
+      [ 4, 0 ],
+      [ 5, 0 ],
+  ]
 
 class CaseBitSelOverPartSelComp:
   class DUT( Component ):
@@ -2139,6 +2196,16 @@ class CaseBitSelOverPartSelComp:
       s.in_ = InPort( Bits32 )
       s.out = OutPort( Bits1 )
       connect( s.out, s.in_[0:4][0] )
+  TV_IN  = _set( 'in_', Bits32, 0 )
+  TV_OUT = _check( 'out', Bits1, 1 )
+  TEST_VECTOR = \
+  [
+      [ 1, 1 ],
+      [ 2, 0 ],
+      [ 3, 1 ],
+      [ 4, 0 ],
+      [ 5, 1 ],
+  ]
 
 class CasePartSelOverBitSelComp:
   class DUT( Component ):
@@ -2146,6 +2213,16 @@ class CasePartSelOverBitSelComp:
       s.in_ = InPort( Bits32 )
       s.out = OutPort( Bits1 )
       connect( s.out, s.in_[1][0:1] )
+  TV_IN  = _set( 'in_', Bits32, 0 )
+  TV_OUT = _check( 'out', Bits1, 1 )
+  TEST_VECTOR = \
+  [
+      [ 1, 0 ],
+      [ 2, 1 ],
+      [ 3, 1 ],
+      [ 4, 0 ],
+      [ 5, 0 ],
+  ]
 
 class CasePartSelOverPartSelComp:
   class DUT( Component ):
@@ -2153,6 +2230,16 @@ class CasePartSelOverPartSelComp:
       s.in_ = InPort( Bits32 )
       s.out = OutPort( Bits1 )
       connect( s.out, s.in_[0:4][0:1] )
+  TV_IN  = _set( 'in_', Bits32, 0 )
+  TV_OUT = _check( 'out', Bits1, 1 )
+  TEST_VECTOR = \
+  [
+      [ 1, 1 ],
+      [ 2, 0 ],
+      [ 3, 1 ],
+      [ 4, 0 ],
+      [ 5, 1 ],
+  ]
 
 #-------------------------------------------------------------------------
 # Test cases that contain SystemVerilog translator errors
